/***********************************************************************

    main.c

    Servo Motor Tester

    2016 (C) Suwa-Koubou


      dsPIC30F3012 pin asign

                  1 MCR#            AVdd 18 -- +5V
   B10K Volume -- 2 AN0/RB0         AVss 17 -- GND
                  3 AN1/RB1      AN6/RB6 16
                  4 AN2/RB2      AN7/RB7 15
                  5 AN3/RB3          Vdd 14 -- +5V
                  6 OSC1             Vss 13 -- GND
                  7 RC15             RB5 12 -- PGEC
   debug  TX  --  8 RC13/U1ATX       RB4 11 -- PGED
          RX  --  9 RC14/U1ARX   OC1/RD0 10 -- PWM out to Servo Motor


     2016/8/21  first release

***********************************************************************/

#include    <p30f3012.h>

#include    "config.h"

void  system_initialize(void);

/*=====================================================================
    main
=====================================================================*/

int  main(void)
{
    system_initialize();
    for(;;);
}


/*--------------------------------------------------------------------
    Timer T2 interrupt
    20msec interval interrupt
--------------------------------------------------------------------*/

void __attribute__ ((__interrupt__, no_auto_psv)) _T2Interrupt(void)
{
    int  data;

    IFS0bits.T2IF = 0;         // clear interrupt flag

    // read potentiometer
    ADCHSbits.CH0SA = 0;       // set channel Zero (AN0)
    ADCON1bits.DONE = 0;       // clear previous done bit
    ADCON1bits.SAMP = 1;       // start sampling 
    while(! ADCON1bits.DONE);  // wait for conversion done
    data = ADCBUF0;

    // set PWM duty
    OC1RS = (long)data  * (long)(1196-184) / (long)0xfff + 184L;


}


/*-------------------------------------------------------------------
    system initialize
-------------------------------------------------------------------*/

void   system_initialize(void)
{

    // OC1(RD0) pwm out
    TRISDbits.TRISD0 = 0;    // RD0 set to digital out
    LATDbits.LATD0 = 0;      // output Low

    // PWM clock by TMR2
    T2CON = 0;
    T2CONbits.TCS = 0;       // internal clock Fosc/4 7.37*16/4=29.48MHz
    T2CONbits.TCKPS = 0b10;  // clock count prescale is 1:64 29.48M/16
    TMR2 = 0;
    PR2 = 9212;              // 29,480,000/64/50=9212.5 (20msec)
    //PR2 = 2303;            // 29,480/64/150=3070.83333 (6.6msec)
    T2CONbits.TON = 1;

    // Servo control PWM
    OC1CON = 0;
    OC1CONbits.OCTSEL = 0;   // clock is TMR2 
    //OC1R = OC1RS = 184;    // 29,480,000/64/2500=184 (0.4ms)
    //OC1R = OC1RS = 230;    // 29,480,000/64/2000=230 (0.5ms)
    //OC1R = OC1RS = 461;    // 29,480,000/64/1000=461 (1.0ms)
    OC1R = OC1RS = 691;      // 29,480,000/64/666.666=691(1.5ms)
    //OC1R = OC1RS = 921;    // 29,480,000/64/500=921 (2.0ms)
    //OC1R = OC1RS = 1151;   // 29,480,000/64/400=1151 (2.5ms)
    //OC1R = OC1RS = 1196;   // 29,480,000/64/385=1196 (2.6ms)
    OC1CONbits.OCM = 0b110;  // PWM start

    // ADC module
    ADCON1bits.ADON = 0;     // AD converter is off
    ADCON1bits.ADSIDL = 0;   // Continue module operation in idle mode
    ADCON1bits.FORM = 0;     // integer (DOUT= 0000 dddd dddd dddd)
    ADCON1bits.SSRC = 0b111; // internal counter ends sampling and starts conversion(auto convert)
    ADCON1bits.ASAM = 0;     // Sampling begins when SAMP bit set

    ADCON2bits.VCFG = 0;     // reference voltage is AVdd, AVss
    ADCON2bits.CSCNA = 0;    // Do not scan inputs 
    ADCON2bits.SMPI = 0;     // interrupt at the cpmpletion of conversion for each sample/convert sequence
    ADCON2bits.BUFM = 0;     // Buffer configured as one 16-word buffer ADCBUF(15...0)
    ADCON2bits.ALTS = 0;     // Always use MUX A input multiplexer setting.

    ADCON3bits.SAMC = 13;    // auto sample time 13Tad
    ADCON3bits.ADRC = 0;     // Clock derived from system clock
    ADCON3bits.ADCS = 3;     // AD conversion clock select, Tad is 4 x Tcy/2

    ADCHSbits.CH0NA = 0;     // Negative input Vref-
    ADCHSbits.CH0SA = 0;     // Positive input AN0
    ADPCFG &= 0xfff0;        // use AN0, AN1, AN2, AN3
    TRISB |= 0x000f;         // RB0-RB3 is input
    ADCSSL = 0x0000;         // skip all channel input scan
    ADCON1bits.ADON = 1;     // turn ADC ON

    //--- setting interrupt ---------------------------------------
    INTCON1 = 0;             // NSTDIS=0
    IEC0bits.T2IE = 1;       // enable T2 interrupt
    IFS0bits.T2IF = 0;       // clear T2 interrupt flag

}


/*** end of main.c ***************************************************/
